from django.contrib.admin.templatetags.admin_modify import *  # noqa: F403 F405 WPS347 E501
from django.contrib.admin.templatetags.admin_modify import (
    change_form_object_tools_tag as base_change_form_object_tools_tag,
)
from django.contrib.admin.templatetags.admin_modify import (
    submit_row_tag as base_submit_row_tag,
)


@register.tag(name="submit_row")  # noqa: F405
def submit_row_tag(parser, token):
    admin_node = base_submit_row_tag(parser, token)
    admin_node.template_name = "jnt_admin_tools/submit_line.html"

    return admin_node


@register.tag(name="change_form_object_tools")  # noqa: F405
def change_form_object_tools_tag(parser, token):
    """Display the row of change form admin tools."""
    admin_node = base_change_form_object_tools_tag(parser, token)
    admin_node.template_name = "jnt_admin_tools/change_form_object_tools.html"

    return admin_node



import abc
import typing as ty

from django.utils.html import format_html


class BaseAdminTool(abc.ABC):
    """Base admin tool."""

    code: str
    title: str
    label_class: str = ""
    target: str = ""

    def __init__(self, model_admin) -> None:
        """Init base admin tool."""
        self.model_admin = model_admin

    @classmethod
    def get_label(cls) -> str:
        """Get label."""
        return cls.title

    @abc.abstractmethod
    def get_url(self, *args, **kwargs) -> str:
        """Get url."""

    def has_permission(self, request) -> bool:
        """Has user permission for view current admin tool."""
        return True


class BaseChangeformTool(BaseAdminTool):
    """Base admin tool."""

    def __init__(self, model_admin, instance) -> None:
        """Init base admin tool."""
        self.instance = instance
        super().__init__(model_admin)

    def get_content(self, request) -> ty.Optional[str]:
        """Get content."""
        url = self.get_url(request)
        if not url:
            return None

        tag_attrs = {
            "href": url,
            "class": self.label_class,
            "target": self.target,
        }

        render_attrs = " ".join(
            (
                '{0}="{1}"'.format(tag_attr, tag_value)
                for tag_attr, tag_value in tag_attrs.items()
                if tag_value
            ),
        )

        return format_html(
            "<a {0}>{1}</a>".format(render_attrs, self.get_label()),
        )


class BaseChangelistTool(BaseAdminTool):
    """Base changelist tool."""

    def _get_content(self, *args, **kwargs) -> ty.Optional[str]:
        """Get content."""
        url = self.get_url(*args, **kwargs)
        if not url:
            return None

        additional_class = "change-list-link-{0}".format(self.code)

        tag_attrs = {
            "href": url,
            "class": "addlink {0}".format(additional_class),
            "target": self.target,
        }

        render_attrs = " ".join(
            (
                '{0}="{1}"'.format(tag_attr, tag_value)
                for tag_attr, tag_value in tag_attrs.items()
                if tag_value
            ),
        )

        return format_html(
            "<a {0}>{1}</a>".format(render_attrs, self.get_label()),
        )



import abc
import typing as ty

from django.contrib import admin
from django.contrib.admin.utils import unquote
from django.contrib.auth.admin import csrf_protect_m
from django.db import models
from django.http import HttpRequest, HttpResponse, HttpResponseRedirect
from django.utils.html import format_html


class BaseChangeformSubmit(abc.ABC):
    """
    Changeform submit button.

    name must start from underscore as django save submits.
    """

    name: str
    title: str
    confirm: str = ""

    def __init__(self, model_admin: admin.ModelAdmin) -> None:
        self.model_admin = model_admin

    def has_permission(
        self,
        request: HttpRequest,
        instance: models.Model,
    ) -> bool:
        return True

    @abc.abstractmethod
    def handle_submit(
        self,
        request: HttpRequest,
        instance: models.Model,
    ) -> None:
        """Handle submit."""

    def get_content(self) -> str:
        onclick = (
            "return confirm('{0}')".format(self.confirm)
            if self.confirm
            else None
        )

        button_attrs = {
            "type": "submit",
            "value": self.title,
            "name": self.name,
            "onclick": onclick,
        }

        return format_html(
            "<input {0}/>".format(
                " ".join(
                    (
                        '{0}="{1}"'.format(tag_attr, tag_value)
                        for tag_attr, tag_value in button_attrs.items()
                        if tag_value
                    ),
                ),
            ),
        )



from tempfile import mktemp
from django.test import TestCase
from django.core import management

from django.urls import reverse
from django.contrib.auth.models import User

from jnt_admin_tools.menu.items import AppList
from jnt_admin_tools.menu.models import Bookmark


class ManagementCommandTest(TestCase):
    def test_custommenu(self):
        # check that custommenu command doesn't raise exceptions
        file_name = mktemp()
        management.call_command("custommenu", file=file_name)
        # and fails if file is already here
        try:
            management.call_command("custommenu", file=file_name)
            assert False
        except:
            pass


class DeleteBookMarkTest(TestCase):
    fixtures = ["users.json"]

    def setUp(self):
        self.superuser_credentials = ("superuser", "123")
        self.staff_credentials = ("staff", "123")
        user = User.objects.get(username=self.superuser_credentials[0])
        self.bookmark = Bookmark.objects.create(
            user=user, url="/test/", title="test"
        )
        self.delete_bookmark_url = reverse(
            "admin-tools-menu-remove-bookmark", args=(self.bookmark.id,)
        )

    def _login_user(self, username, password):
        try:
            user = User.objects.get(username=username)
            self.client.force_login(user)
        except AttributeError:  # in Django<1.9
            logged_in = self.client.login(
                username=user.username, password=password
            )
            self.assertTrue(logged_in)

    def test_removing_of_own_bookmark(self):
        self.assertEqual(Bookmark.objects.count(), 1)
        self._login_user(*self.superuser_credentials)
        self.client.post(self.delete_bookmark_url)
        self.assertFalse(Bookmark.objects.count())

    def test_removing_others_bookmark(self):
        self.assertEqual(Bookmark.objects.count(), 1)
        self._login_user(*self.staff_credentials)
        self.client.post(self.delete_bookmark_url)
        self.assertEqual(Bookmark.objects.first(), self.bookmark)


__test__ = {
    "AppList.is_empty": AppList.is_empty,
}



from django.core.checks import Warning, register
from django.template.loader import TemplateDoesNotExist, get_template

W001 = Warning(
    'You must add "jnt_admin_tools.template_loaders.Loader" in your '
    "template loaders variable, see: "
    "https://django-admin-tools.readthedocs.io/en/latest/configuration.html",
    id="jnt_admin_tools.W001",
    obj="jnt_admin_tools",
)


@register("jnt_admin_tools")
def check_admin_tools_configuration(app_configs=None, **kwargs):
    result = []
    try:
        get_template("admin:admin/base.html")
    except TemplateDoesNotExist:
        result.append(W001)
    return result



from django.contrib.admin.templatetags.admin_list import *  # noqa: F403 WPS347
from django.contrib.admin.templatetags.admin_list import (
    change_list_object_tools_tag as base_change_list_object_tools_tag,
)


@register.tag(name="change_list_object_tools")  # noqa: F405
def change_list_object_tools_tag(parser, token):
    """Display the row of change list admin tools."""
    admin_node = base_change_list_object_tools_tag(parser, token)
    admin_node.template_name = "jnt_admin_tools/change_list_object_tools.html"
    return admin_node



"""
Based on https://bitbucket.org/tzulberti/django-apptemplates/
and http://djangosnippets.org/snippets/1376/

Django template loader that allows you to load a template from a
specific application. This allows you to both extend and override
a template at the same time. The default Django loaders require you
to copy the entire template you want to override, even if you only
want to override one small block.

Template usage example::
    {% extends "admin:admin/base.html" %}
"""

from os.path import join

from django.apps import apps
from django.template.loaders.filesystem import Loader as FilesystemLoader

_cache = {}


def get_app_template_dir(app_name):
    """Get the template directory for an application

    Uses apps interface available in django 1.7+

    Returns a full path, or None if the app was not found.
    """
    if app_name in _cache:
        return _cache[app_name]
    template_dir = None
    for app in apps.get_app_configs():
        if app.label == app_name:
            template_dir = join(app.path, "templates")
            break
    _cache[app_name] = template_dir
    return template_dir


class Loader(FilesystemLoader):
    is_usable = True

    def get_template_sources(self, template_name, template_dirs=None):
        """
        Returns the absolute paths to "template_name" in the specified app.
        If the name does not contain an app name (no colon), an empty list
        is returned.
        The parent FilesystemLoader.load_template_source() will take care
        of the actual loading for us.
        """
        if ":" not in template_name:
            return []
        app_name, template_name = template_name.split(":", 1)
        template_dir = get_app_template_dir(app_name)
        if template_dir:
            try:
                from django.template import Origin

                origin = Origin(
                    name=join(template_dir, template_name),
                    template_name=template_name,
                    loader=self,
                )
            except (ImportError, TypeError):
                origin = join(template_dir, template_name)
            return [origin]
        return []



"""
Menu utilities.
"""

from django.conf import settings

try:
    from importlib import import_module
except ImportError:
    # Django < 1.9 and Python < 2.7
    from django.utils.importlib import import_module
try:
    from django.urls import reverse
except ImportError:
    from django.core.urlresolvers import reverse


def _get_menu_cls(menu_cls, context):
    if isinstance(menu_cls, dict):
        curr_url = context.get("request").path
        for key in menu_cls:
            admin_site_mod, admin_site_inst = key.rsplit(".", 1)
            admin_site_mod = import_module(admin_site_mod)
            admin_site = getattr(admin_site_mod, admin_site_inst)
            admin_url = reverse("%s:index" % admin_site.name)
            if curr_url.startswith(admin_url):
                mod, inst = menu_cls[key].rsplit(".", 1)
                mod = import_module(mod)
                return getattr(mod, inst)
    else:
        mod, inst = menu_cls.rsplit(".", 1)
        mod = import_module(mod)
        return getattr(mod, inst)
    raise ValueError('Dashboard menu matching "%s" not found' % menu_cls)


def get_admin_menu(context):
    """
    Returns the admin menu defined by the user or the default one.
    """
    return _get_menu_cls(
        getattr(
            settings, "ADMIN_TOOLS_MENU", "jnt_admin_tools.menu.DefaultMenu"
        ),
        context,
    )()



"""
Dashboard template tags, the following dashboard tags are available:
 * ``{% admin_tools_render_dashboard %}``
 * ``{% admin_tools_render_dashboard_module %}``
 * ``{% admin_tools_render_dashboard_css %}``

To load the dashboard tags: ``{% load admin_tools_dashboard_tags %}``.
"""

import math

from django import template
from django.db import IntegrityError
from django.urls import reverse
from jnt_admin_tools.dashboard.models import DashboardPreferences
from jnt_admin_tools.dashboard.utils import get_dashboard
from jnt_admin_tools.utils import get_admin_site_name

register = template.Library()
tag_func = register.inclusion_tag(
    "jnt_admin_tools/dashboard/dummy.html", takes_context=True
)


def admin_tools_render_dashboard(context, location="index", dashboard=None):
    """
    Template tag that renders the dashboard, it takes two optional arguments:

    ``location``
        The location of the dashboard, it can be 'index' (for the admin index
        dashboard) or 'app_index' (for the app index dashboard), the default
        value is 'index'.

    ``dashboard``
        An instance of ``Dashboard``, if not given, the dashboard is retrieved
        with the ``get_index_dashboard`` or ``get_app_index_dashboard``
        functions, depending on the ``location`` argument.
    """
    if dashboard is None:
        dashboard = get_dashboard(context, location)

    dashboard.init_with_context(context)
    dashboard._prepare_children()

    preferences = "{}"  # noqa: P103

    try:
        preferences = DashboardPreferences.objects.get(
            user=context["request"].user, dashboard_id=dashboard.get_id()
        ).data
    except DashboardPreferences.DoesNotExist:
        try:
            DashboardPreferences.objects.create(
                user=context["request"].user,
                dashboard_id=dashboard.get_id(),
                data=preferences,  # noqa: P103
            )
        except IntegrityError:
            # dashboard already was saved for that (user, dashboard)
            pass

    context.update(
        {
            "template": dashboard.template,
            "dashboard": dashboard,
            "dashboard_preferences": preferences,
            "split_at": math.ceil(
                float(len(dashboard.children)) / float(dashboard.columns)
            ),
            "has_disabled_modules": len(
                [m for m in dashboard.children if not m.enabled]
            )
            > 0,
            "admin_url": reverse("%s:index" % get_admin_site_name(context)),
        }
    )
    return context


admin_tools_render_dashboard = tag_func(admin_tools_render_dashboard)


def admin_tools_render_dashboard_module(context, module):
    """
    Template tag that renders a given dashboard module, it takes a
    ``DashboardModule`` instance as first parameter.
    """
    module.init_with_context(context)
    context.update(
        {
            "template": module.template,
            "module": module,
            "admin_url": reverse("%s:index" % get_admin_site_name(context)),
        }
    )
    return context


admin_tools_render_dashboard_module = tag_func(
    admin_tools_render_dashboard_module
)


def admin_tools_render_dashboard_css(
    context, location="index", dashboard=None
):
    """
    Template tag that renders the dashboard css files, it takes two optional
    arguments:

    ``location``
        The location of the dashboard, it can be 'index' (for the admin index
        dashboard) or 'app_index' (for the app index dashboard), the default
        value is 'index'.

    ``dashboard``
        An instance of ``Dashboard``, if not given, the dashboard is retrieved
        with the ``get_index_dashboard`` or ``get_app_index_dashboard``
        functions, depending on the ``location`` argument.
    """
    if dashboard is None:
        dashboard = get_dashboard(context, location)

    context.update(
        {
            "template": "jnt_admin_tools/dashboard/css.html",
            "css_files": dashboard.Media.css,
        }
    )
    return context


admin_tools_render_dashboard_css = tag_func(admin_tools_render_dashboard_css)



import os
from django.core.management.base import BaseCommand, CommandError
from django.template.loader import render_to_string

DEFAULT_FILE = "dashboard.py"


class Command(BaseCommand):
    help = (
        "Creates a template file containing the base code to get you "
        "started with your custom dashboard."
    )
    args = "[file]"
    label = "application name"

    def handle(self, file=None, **options):
        context = {}
        context["project"] = os.path.basename(os.getcwd())
        tpl = [
            "dashboard/dashboard.txt",
            "jnt_admin_tools/dashboard/dashboard.txt",
        ]
        dst = file is not None and file or DEFAULT_FILE
        if os.path.exists(dst):
            raise CommandError('file "%s" already exists' % dst)
        context["file"] = os.path.basename(dst).split(".")[0]
        open(dst, "w").write(render_to_string(tpl, context))
        self.stdout.write('"%s" written.' % os.path.join(dst))



import abc
import typing as ty

from django.contrib import admin
from django.contrib.admin.utils import unquote
from django.contrib.auth.admin import csrf_protect_m
from django.db import models
from django.http import HttpRequest, HttpResponse, HttpResponseRedirect
from django.utils.html import format_html

from jnt_admin_tools.admin.changeform_submits_set import ChangeformSubmitsSet


class ChangeFormSubmitsMixin:
    changeform_submits = ()

    def render_change_form(
        self,
        request,
        context,
        add=False,
        change=False,
        form_url="",
        obj=None,  # noqa: WPS110
    ):

        context["changeform_submits"] = ChangeformSubmitsSet(
            request,
            self,
            obj,
        )

        return super().render_change_form(
            request,
            context,
            add,
            change,
            form_url,
            obj,
        )

    def get_exclude_changeform_submits(self, request):
        return ()

    def get_changeform_submits(self, request, instance):  # noqa: WPS615
        return self.changeform_submits

    @csrf_protect_m
    def changeform_view(
        self,
        request,
        object_id=None,
        form_url="",
        extra_context=None,
    ):
        if request.method == "POST" and object_id:
            instance = self.get_object(request, unquote(object_id))
            response_view = ChangeformSubmitsSet(
                request,
                self,
                instance,
            ).handle_submit(instance)
            if response_view:
                return response_view

        return super().changeform_view(
            request,
            object_id,
            form_url,
            extra_context,
        )



from jnt_admin_tools.admin.changeform_tools_set import ChangeformToolsSet
from jnt_admin_tools.admin.changelist_tools_set import ChangelistToolsSet


class AdminToolsMixin:
    changeform_tools = ()
    changelist_tools = ()

    def render_change_form(
        self,
        request,
        context,
        add=False,
        change=False,
        form_url="",
        obj=None,  # noqa: WPS110
    ):
        if change:
            context.update(
                changeform_tools=ChangeformToolsSet(request, self, obj),
            )

        return super().render_change_form(
            request,
            context,
            add,
            change,
            form_url,
            obj,
        )

    def changelist_view(self, request, extra_context=None):
        extra_context = extra_context or {}
        extra_context.update(
            changelist_tools=ChangelistToolsSet(request, self),
        )
        return super().changelist_view(request, extra_context=extra_context)

    def get_changeform_tools(self):  # noqa: WPS615
        return self.changeform_tools

    def get_changelist_tools(self):  # noqa: WPS615
        return self.changelist_tools



from jnt_admin_tools.menu.items import MenuItem


class AdminMenuItem(MenuItem):
    """Admin menu item."""

    def __init__(self, title, menu_items):
        """Initializing AdminMenuItem."""
        super().__init__(title)
        self.menu_items = menu_items

    def init_with_context(self, context):
        """Init menu items."""
        for title, url, perm in self.menu_items:
            if perm and not context.request.user.has_perm(perm):
                continue

            self.children.append(MenuItem(title, url))



import typing as ty

from django.contrib import admin
from django.db import models
from django.http import HttpRequest, HttpResponse, HttpResponseRedirect

from jnt_admin_tools.admin.base_changeform_submit import BaseChangeformSubmit


class ChangeformSubmitsSet:
    def __init__(
        self,
        request: HttpRequest,
        model_admin: admin.ModelAdmin,
        instance: models.Model,
    ) -> None:
        self.request = request
        self.model_admin = model_admin
        self.instance = instance
        self._submits = []

    def handle_submit(
        self,
        instance: models.Model,
    ) -> ty.Optional[HttpResponse]:
        for submit in self._get_submits(instance):
            if submit.name in self.request.POST:
                response = submit.handle_submit(self.request, instance)
                return response or HttpResponseRedirect(self.request.path)

    def __iter__(self) -> ty.Iterator[str]:
        yield from (
            submit.get_content() for submit in self._get_submits(self.instance)
        )

    def _get_submits(
        self,
        instance: models.Model,
    ) -> ty.List[BaseChangeformSubmit]:
        if not self._submits:
            excluded = self.model_admin.get_exclude_changeform_submits(
                self.request,
            )
            changeform_submits = self.model_admin.get_changeform_submits(
                self.request,
                instance,
            )
            for submit_class in changeform_submits:
                if submit_class in excluded:
                    continue
                submit = submit_class(self.model_admin)
                if submit.has_permission(self.request, instance):
                    self._submits.append(submit)

        return self._submits



class ChangeformToolsSet:
    def __init__(self, request, model_admin, instance) -> None:
        self.request = request
        self.model_admin = model_admin
        self.instance = instance

    def __str__(self) -> str:
        return "{0}: changeform admin tools".format(self.model_admin)

    def __iter__(self):
        for changeform_tool_class in self.model_admin.get_changeform_tools():
            changeform_tool = changeform_tool_class(
                self.model_admin,
                self.instance,
            )

            if not changeform_tool.has_permission(self.request):
                continue

            changeform_tool_content = changeform_tool.get_content(
                self.request,
            )
            if not changeform_tool_content:
                continue

            yield changeform_tool_content



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
from django.conf import settings


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='Bookmark',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('url', models.CharField(max_length=255)),
                ('title', models.CharField(max_length=255)),
                ('user', models.ForeignKey(to=settings.AUTH_USER_MODEL, on_delete=models.CASCADE)),
            ],
            options={
                'ordering': ('id',),
                'db_table': 'admin_tools_menu_bookmark',
            },
            bases=(models.Model,),
        ),
    ]



from django.urls import path
from jnt_admin_tools.menu import views

urlpatterns = [
    path(
        "add_bookmark/",
        views.add_bookmark,
        name="admin-tools-menu-add-bookmark",
    ),
    path(
        "edit_bookmark/<id>/",
        views.edit_bookmark,
        name="admin-tools-menu-edit-bookmark",
    ),
    path(
        "remove_bookmark/<id>/",
        views.remove_bookmark,
        name="admin-tools-menu-remove-bookmark",
    ),
]



class ChangelistToolsSet:
    def __init__(self, request, model_admin) -> None:
        self.request = request
        self.model_admin = model_admin

    def __str__(self) -> str:
        return "{0}: changelist admin tools".format(self.model_admin)

    def __iter__(self):
        for changelist_tool_class in self.model_admin.get_changelist_tools():
            changelist_tool = changelist_tool_class(self.model_admin)

            if not changelist_tool.has_permission(self.request):
                continue

            changelist_tool_content = changelist_tool._get_content(
                self.request,
            )
            if not changelist_tool_content:
                continue

            yield changelist_tool_content
